#!/bin/bash

#-- configuration
TMP_STORE=/tmp/csvstore
BACKUP_DIR=/home/backup

#--

if test $# -eq 0
 then
   echo "usage $0 <dbname> [dbcredentials]"
   exit 0
 else
   # pass database as 1st param
   db=$1
   MYSQL_CMD="/usr/bin/mysql -D $@"
   MYSQL_DUMP="/usr/bin/mysqldump $@"
 fi

BKP_PATH=$BACKUP_DIR/$db

if [ ! -d $BKP_PATH ]
  then
    mkdir -p $BKP_PATH
fi

if [ ! -d $TMP_STORE ]
 then
   mkdir -p $TMP_STORE
 else
   rm -rf $TMP_STORE/*
fi

#make sure mysql can write the the folder
chmod 1777 $TMP_STORE
     
dumpcsv(){
 $MYSQL_CMD -D $db -e "show tables" | grep -v "Tables_in" > $TMP_STORE/tables.lst
 cat $TMP_STORE/tables.lst | while read tablename
    do
     echo "SELECT * INTO OUTFILE '$TMP_STORE/${tablename}.csv' FIELDS TERMINATED BY '\t' FROM $tablename;"
    done > /dev/shm/sqlcmd.sql
 $MYSQL_CMD -D $db < /dev/shm/sqlcmd.sql
}

dumpstruct(){
 $MYSQL_DUMP --routines --triggers --no-data --compact > $TMP_STORE/struct.sql
}

CWD=`/bin/pwd`

#check if we are running on a sunday
if test `date +%w` -eq 0; then
  #do we have a previous week folder 
  if [ -d "$BKP_PATH/Week_3" ] ; then
    #do we have a week before in tar.bz2 format
    if [ -f "$BKP_PATH/Week_2.tar.bz2" ] ; then
       #do we have a week before last in tar.bz2 format
       if [ -f "$BKP_PATH/Week_1.tar.bz2" ] ; then
          #we dont need the very old backup
          rm -rf "$BKP_PATH/Week_1.tar.bz2"
       fi
       #rename the last week to before last week
       mv -f "$BKP_PATH/Week_2.tar.bz2" "$BKP_PATH/Week_1.tar.bz2"
    fi
    #create a tar.bz2 file of the immediate past week.
    cd $BKP_PATH
    /bin/tar -cjf "Week_2.tar.bz2" Week_3/*
    cd $CWD
    rm -rf $BKP_PATH/Week_3
  fi
fi


# take dump if we are running..
dumpcsv
dumpstruct

# if we dont have the current directory.. we are either running first time or
# running date is sunday, so take a full backup

if [ ! -d "$BKP_PATH/Week_3" ] ; then
   mkdir -p "$BKP_PATH/Week_3/Full"
   mv $TMP_STORE/* $BKP_PATH/Week_3/Full/
else
  #-- we are now in the week day, subsequent runs need only the diff.
  CWD=`/bin/pwd`
  WeekDay=`date +%a`
  if [ ! -d $BKP_PATH/Week_3/$WeekDay ]
   then
      mkdir $BKP_PATH/Week_3/$WeekDay
      cd $TMP_STORE

      for i in *
        do
         diff $BKP_PATH/Week_3/Full/$i $i > /dev/shm/diff.txt
         if test -s /dev/shm/diff.txt
           then
           mv /dev/shm/diff.txt $BKP_PATH/Week_3/$WeekDay/${i}.diff
     fi
        done
   fi

  cd $CWD
fi


exit 0

